Struct CommandQueue
pub struct CommandQueue { /* private fields */ }Expand description
Densely and efficiently stores a queue of heterogenous types implementing Command.
Implementations
impl CommandQueue
impl CommandQueue
pub fn apply(&mut self, world: &mut World)
pub fn apply(&mut self, world: &mut World)
Execute the queued Commands in the world after applying any commands in the world’s internal queue.
This clears the queue.
pub fn append(&mut self, other: &mut CommandQueue)
pub fn append(&mut self, other: &mut CommandQueue)
Take all commands from other and append them to self, leaving other empty