Struct EntityFetcher
pub struct EntityFetcher<'w> { /* private fields */ }Expand description
Provides a safe interface for non-structural access to the entities in a World.
This cannot add or remove components, or spawn or despawn entities,
making it relatively safe to access in concert with other ECS data.
This type can be constructed via World::entities_and_commands,
or DeferredWorld::entities_and_commands.
Implementations
impl<'w> EntityFetcher<'w>
impl<'w> EntityFetcher<'w>
pub fn get<F>(
&self,
entities: F,
) -> Result<<F as WorldEntityFetch>::Ref<'_>, EntityNotSpawnedError>where
F: WorldEntityFetch,
pub fn get<F>(
&self,
entities: F,
) -> Result<<F as WorldEntityFetch>::Ref<'_>, EntityNotSpawnedError>where
F: WorldEntityFetch,
Returns EntityRefs that expose read-only operations for the given
entities, returning Err if any of the given entities do not exist.
This function supports fetching a single entity or multiple entities:
- Pass an
Entityto receive a singleEntityRef. - Pass a slice of
Entitys to receive aVec<EntityRef>. - Pass an array of
Entitys to receive an equally-sized array ofEntityRefs. - Pass a reference to a
EntityHashSetto receive anEntityHashMap<EntityRef>.
Errors
If any of the given entities do not exist in the world, the first
Entity found to be missing will return an EntityNotSpawnedError.
Examples
For examples, see World::entity.
pub fn get_mut<F>(
&mut self,
entities: F,
) -> Result<<F as WorldEntityFetch>::DeferredMut<'_>, EntityMutableFetchError>where
F: WorldEntityFetch,
pub fn get_mut<F>(
&mut self,
entities: F,
) -> Result<<F as WorldEntityFetch>::DeferredMut<'_>, EntityMutableFetchError>where
F: WorldEntityFetch,
Returns EntityMuts that expose read and write operations for the
given entities, returning Err if any of the given entities do not
exist.
This function supports fetching a single entity or multiple entities:
- Pass an
Entityto receive a singleEntityMut.- This reference type allows for structural changes to the entity, such as adding or removing components, or despawning the entity.
- Pass a slice of
Entitys to receive aVec<EntityMut>. - Pass an array of
Entitys to receive an equally-sized array ofEntityMuts. - Pass a reference to a
EntityHashSetto receive anEntityHashMap<EntityMut>.
Errors
- Returns
EntityMutableFetchError::NotSpawnedif any of the givenentitiesdo not exist in the world.- Only the first entity found to be missing will be returned.
- Returns
EntityMutableFetchError::AliasedMutabilityif the same entity is requested multiple times.
Examples
For examples, see DeferredWorld::entity_mut.