Struct FilteredEntityMut
pub struct FilteredEntityMut<'w> { /* private fields */ }Expand description
Provides mutable access to a single entity and some of its components defined by the contained Access.
Implementations
impl<'w> FilteredEntityMut<'w>
impl<'w> FilteredEntityMut<'w>
pub fn reborrow(&mut self) -> FilteredEntityMut<'_>
pub fn reborrow(&mut self) -> FilteredEntityMut<'_>
Returns a new instance with a shorter lifetime.
This is useful if you have &mut FilteredEntityMut, but you need FilteredEntityMut.
pub fn as_readonly(&self) -> FilteredEntityRef<'_>
pub fn as_readonly(&self) -> FilteredEntityRef<'_>
Gets read-only access to all of the entity’s components.
pub fn location(&self) -> EntityLocation
pub fn location(&self) -> EntityLocation
Gets metadata indicating the location where the current entity is stored.
pub fn components(&self) -> impl Iterator<Item = ComponentId>
pub fn components(&self) -> impl Iterator<Item = ComponentId>
Returns an iterator over the component ids that are accessed by self.
pub fn access(&self) -> &Access<ComponentId>
pub fn access(&self) -> &Access<ComponentId>
Returns a reference to the underlying Access.
pub fn contains<T>(&self) -> boolwhere
T: Component,
pub fn contains<T>(&self) -> boolwhere
T: Component,
Returns true if the current entity has a component of type T.
Otherwise, this returns false.
Notes
If you do not know the concrete type of a component, consider using
Self::contains_id or Self::contains_type_id.
pub fn contains_id(&self, component_id: ComponentId) -> bool
pub fn contains_id(&self, component_id: ComponentId) -> bool
Returns true if the current entity has a component identified by component_id.
Otherwise, this returns false.
Notes
- If you know the concrete type of the component, you should prefer
Self::contains. - If you know the component’s
TypeIdbut not itsComponentId, consider usingSelf::contains_type_id.
pub fn contains_type_id(&self, type_id: TypeId) -> bool
pub fn contains_type_id(&self, type_id: TypeId) -> bool
Returns true if the current entity has a component with the type identified by type_id.
Otherwise, this returns false.
Notes
- If you know the concrete type of the component, you should prefer
Self::contains. - If you have a
ComponentIdinstead of aTypeId, consider usingSelf::contains_id.
pub fn get<T>(&self) -> Option<&T>where
T: Component,
pub fn get<T>(&self) -> Option<&T>where
T: Component,
Gets access to the component of type T for the current entity.
Returns None if the entity does not have a component of type T.
pub fn get_ref<T>(&self) -> Option<Ref<'_, T>>where
T: Component,
pub fn get_ref<T>(&self) -> Option<Ref<'_, T>>where
T: Component,
Gets access to the component of type T for the current entity,
including change detection information as a Ref.
Returns None if the entity does not have a component of type T.
pub fn get_mut<T>(&mut self) -> Option<Mut<'_, T>>where
T: Component,
pub fn get_mut<T>(&mut self) -> Option<Mut<'_, T>>where
T: Component,
Gets mutable access to the component of type T for the current entity.
Returns None if the entity does not have a component of type T.
pub fn into_mut<T>(self) -> Option<Mut<'w, T>>where
T: Component,
pub fn into_mut<T>(self) -> Option<Mut<'w, T>>where
T: Component,
Consumes self and gets mutable access to the component of type T
with the world 'w lifetime for the current entity.
Returns None if the entity does not have a component of type T.
pub fn get_change_ticks<T>(&self) -> Option<ComponentTicks>where
T: Component,
pub fn get_change_ticks<T>(&self) -> Option<ComponentTicks>where
T: Component,
Retrieves the change ticks for the given component. This can be useful for implementing change detection in custom runtimes.
pub fn get_change_ticks_by_id(
&self,
component_id: ComponentId,
) -> Option<ComponentTicks>
pub fn get_change_ticks_by_id( &self, component_id: ComponentId, ) -> Option<ComponentTicks>
Retrieves the change ticks for the given ComponentId. This can be useful for implementing change
detection in custom runtimes.
You should prefer to use the typed API Self::get_change_ticks where possible and only
use this in cases where the actual component types are not known at
compile time.
pub fn get_by_id(&self, component_id: ComponentId) -> Option<Ptr<'_>>
pub fn get_by_id(&self, component_id: ComponentId) -> Option<Ptr<'_>>
Gets the component of the given ComponentId from the entity.
You should prefer to use the typed API Self::get where possible and only
use this in cases where the actual component types are not known at
compile time.
Unlike FilteredEntityMut::get, this returns a raw pointer to the component,
which is only valid while the FilteredEntityMut is alive.
pub fn get_mut_by_id(
&mut self,
component_id: ComponentId,
) -> Option<MutUntyped<'_>>
pub fn get_mut_by_id( &mut self, component_id: ComponentId, ) -> Option<MutUntyped<'_>>
Gets a MutUntyped of the component of the given ComponentId from the entity.
You should prefer to use the typed API Self::get_mut where possible and only
use this in cases where the actual component types are not known at
compile time.
Unlike FilteredEntityMut::get_mut, this returns a raw pointer to the component,
which is only valid while the FilteredEntityMut is alive.